import os

import girok.api.auth as auth_api
from girok.constants import APP_DIR, CONFIG_PATH
from girok.utils.json_utils import read_json, update_json, write_json


class AuthHandler:

    @classmethod
    def init(cls) -> None:
        # Ensure application directory exists
        if not os.path.isdir(APP_DIR):
            os.makedirs(APP_DIR)

        # Ensure config.json exists
        if not os.path.exists(CONFIG_PATH):
            write_json(CONFIG_PATH, {})

    @classmethod
    def is_logged_in(cls) -> bool:
        # Ensure config.json exists
        if not is_config_exist():
            return False

        # Ensure access_token is present
        cfg = read_json(CONFIG_PATH)
        if "access_token" not in cfg:
            return False

        # Ensure access_token is valid
        access_token = cfg["access_token"]
        return auth_api.verify_access_token(access_token)

    @classmethod
    def login(cls, access_token: str) -> None:
        update_json(CONFIG_PATH, {"access_token": access_token})

    @classmethod
    def logout(cls) -> None:
        cfg = read_json(CONFIG_PATH)
        if "access_token" in cfg:
            del cfg["access_token"]
            write_json(CONFIG_PATH, cfg)

    @classmethod
    def get_access_token(cls) -> str:
        cfg = read_json(CONFIG_PATH)
        if "access_token" not in cfg:
            raise ValueError("Access token not found.")
        return cfg["access_token"]


def is_config_exist():
    return os.path.exists(CONFIG_PATH)



from typing import Optional

from rich import print
from rich.console import Console
from rich.style import Style
from rich.text import Text
from rich.tree import Tree

from girok.constants import (
    CATEGORY_COLOR_AUTO_ASSIGNMENT_ORDER,
    CATEGORY_COLOR_PALETTE,
    CONFIG_PATH,
    DEFAULT_CATEGORY_TEXT_COLOR,
    HIGHLIGHT_CATEGORY_TEXT_COLOR,
    DisplayBoxType,
    Emoji,
)
from girok.utils.json_utils import read_json, update_json

console = Console()


def display_categories_tree(root_categories: list[dict], highlight_category_path: Optional[str] = None):
    """Display the category tree

    Args:
        root_categories (list[dict]): List of top-level categories.
        highlight_category_path (Optional[str], optional): Category path name to be highlighted. Must be in 'A/B/C' format. Defaults to None.
    """
    tree = Tree("")

    for category in root_categories:
        display_category_subtree(
            tree=tree,
            category=category,
            highlight_category_path=highlight_category_path,
        )
    console.print(tree)


def display_category_subtree(
    tree: Tree,
    category: dict,
    highlight_category_path: Optional[str] = None,
    parent_cumul_path: str = "",
):
    """Display the subtree of a single tree node.

    Args:
        tree (Tree): rich.tree.Tree object.
        category (dict): A single category. It's a dictionary with keys 'id', 'name', 'color', 'children'
        highlight_category_path (Optional[str], optional): Category path name to be highlighted. Must be in 'A/B/C' format. Defaults to None.
        parent_cumul_path (str, optional): The cumulative category path string of the current node's parent. Defaults to "".
    """
    category_name = category["name"]
    category_color = category["color"]
    category_children = category["children"]
    current_category_path = f"{parent_cumul_path}/{category_name}".lstrip("/")  # A/B/C

    circle_text = Text(text=Emoji.CIRCLE, style=Style(color=CATEGORY_COLOR_PALETTE[category_color]))

    category_name_text = Text(
        text=f"{category_name}",
        style=Style(
            color=(
                DEFAULT_CATEGORY_TEXT_COLOR
                if highlight_category_path != current_category_path
                else HIGHLIGHT_CATEGORY_TEXT_COLOR
            )
        ),
    )

    item_text = Text.assemble(circle_text, " ", category_name_text)
    sub_tree = tree.add(item_text)
    for child in category_children:
        display_category_subtree(
            tree=sub_tree,
            category=child,
            highlight_category_path=highlight_category_path,
            parent_cumul_path=current_category_path,
        )


def display_category_color_palette() -> None:
    tree = Tree("Supported category colors")
    for color_name, hex in CATEGORY_COLOR_PALETTE.items():
        circle_text = Text(text=Emoji.CIRCLE, style=Style(color=CATEGORY_COLOR_PALETTE[color_name]))
        category_name_text = Text(text=f"{color_name} ({hex})", style=Style(color=DEFAULT_CATEGORY_TEXT_COLOR))
        item_text = Text.assemble(circle_text, " ", category_name_text)
        tree.add(item_text)
    console.print(tree)


def get_next_category_color() -> str:
    cfg = read_json(CONFIG_PATH)

    if "next_category_color_idx" in cfg:
        next_category_color_idx = cfg["next_category_color_idx"]
    else:
        next_category_color_idx = 0

    next_category_color = CATEGORY_COLOR_AUTO_ASSIGNMENT_ORDER[next_category_color_idx]
    next_category_color_idx = (next_category_color_idx + 1) % len(CATEGORY_COLOR_AUTO_ASSIGNMENT_ORDER)
    update_json(CONFIG_PATH, {"next_category_color_idx": next_category_color_idx})
    return next_category_color



import json
import os


def read_json(path: str) -> dict:
    with open(path) as f:
        data = json.load(f)
        return data


def write_json(path: str, data: dict):
    with open(path, "w") as f:
        json.dump(data, f)


def update_json(path: str, data: dict):
    with open(path) as f:
        org_data = json.load(f)
        org_data.update(**data)
    write_json(path, org_data)



import calendar
from datetime import datetime

from rich.style import Style
from rich.text import Text
from textual import log
from textual.widgets.tree import TreeNode

from girok.constants import Emoji

 
def build_category_tree(tree_node: TreeNode, categories: list[dict]) -> None:
    for category in categories:
        if not category['children']:
            cur = tree_node.add_leaf(category['name'], data={"color": category['color'], 'id': category['id']})
        else:
            cur = tree_node.add(category['name'], expand=True, data={"color": category['color'], 'id': category['id']})
            build_category_tree(cur, category['children'])


def get_full_path_from_node(node):
    label = str(node._label)
    if label.endswith(" " + Emoji.LEFT_ARROW):
        label = label[:-2]
    else:
        pass
    if label == "All Categories":
        return ""
    elif label == "No Category":
        return "No Category"
    node_name = label
    parent_path = get_full_path_from_node(node.parent)
    return parent_path + node_name + "/"


def convert_day_to_cell_num(year: int, month: int, day: int):
    """ """
    first_weekday, total_days = calendar.monthrange(year, month)
    return first_weekday + day - 1


def convert_cell_num_to_day(year: int, month: int, cell_num: int):
    first_weekday, total_days = calendar.monthrange(year, month)
    return cell_num - first_weekday + 1


def convert_cell_num_to_coord(cell_num: int):
    """
    cell_num: 0 ~ 34
    """

    return (cell_num // 7, cell_num % 7)


def convert_coord_to_cell_num(x: int, y: int):
    return x * 7 + y


def get_date_obj_from_str_separated_by_T(s: str):
    s = str(s)
    delim = "T" if "T" in s else " "
    return datetime.strptime(s, f"%Y-%m-%d{delim}%H:%M:%S")


def remove_left_arrow(cell):
    if cell is None:
        return
    if not cell.children:
        return
    cell_label = cell.children[0]
    cell_label_text = cell_label.render()
    style = cell_label_text.style
    if str(cell_label_text).endswith(" " + Emoji.LEFT_ARROW):
        if cell_label_text.spans:
            style = cell_label_text.spans[0].style

    if str(cell_label_text).endswith(" " + Emoji.LEFT_ARROW):
        new_label_text = Text(str(cell_label_text)[:2])
    else:
        new_label_text = Text(str(cell_label_text))
    cell_label.update(new_label_text)


def add_left_arrow(cell):
    if not cell.children:
        return

    cell_label = cell.children[0]
    cell_label_text = cell_label.render()
    style = cell_label_text.style

    if str(cell_label_text).endswith(" " + Emoji.LEFT_ARROW):
        new_label_text = Text(str(cell_label_text), style=style)
    else:
        new_label_text = Text.assemble(
            Text(str(cell_label_text), style=style),
            " ",
            Text(Emoji.LEFT_ARROW, style=Style(color="#9bdfbb")),
        )
    cell_label.update(new_label_text)


def remove_left_arrow_tree(node):
    label_text = node.label
    style = label_text.style
    if str(label_text).endswith(" " + Emoji.LEFT_ARROW):
        new_label_text = Text(str(label_text)[:-2], style=style)
    else:
        new_label_text = Text(str(label_text), style=style)

    node.set_label(new_label_text)


def add_left_arrow_tree(node):
    label_text = node.label
    style = label_text.style
    if str(label_text).endswith(" " + Emoji.LEFT_ARROW):
        pass
    else:
        node.set_label(
            Text.assemble(label_text, " ", Emoji.LEFT_ARROW, style=style)
        )


def remove_highlight(node):
    label_text = node.label
    style = label_text.style
    node.set_label(Text(str(label_text)))


def add_highlight(node):
    label = str(node.label)
    node.set_label(Text(label, style=Style(color="#9bdfbb")))



import typer
import girok.calendar_cli.calendar_main as calendar_main

app = typer.Typer(rich_markup_mode="rich")


@app.command(
    "cal",
    help="[green]Open Calendar GUI[/green]",
    rich_help_panel=":tear-off_calendar: [bold yellow1]Calendar Commands[/bold yellow1]"
)
def show_calendar():
    cal_app = calendar_main.Entry()
    cal_app.run()


import calendar
from datetime import datetime

from rich import box
from rich.style import Style
from rich.table import Column, Table
from rich.text import Text
from textual import log
from textual.app import App, ComposeResult
from textual.containers import Container, Horizontal, Vertical
from textual.messages import Message
from textual.reactive import reactive, var
from textual.scroll_view import ScrollView
from textual.widget import Widget
from textual.widgets import Button, Footer, Header, Label, Placeholder, Static, Tree

import girok.api.category as category_api
import girok.calendar_cli.utils as calendar_utils

from girok.calendar_cli.calendar_app import CalendarApp
from girok.calendar_cli.calendar_container import Calendar, CalendarContainer
from girok.calendar_cli.sidebar import CategoryTree, SidebarContainer, TagTree
from girok.commands.task.display import display_events_by_list
from girok.constants import TABLE_HEADER_DATE_COLOR


class Entry(App):
    CSS_PATH = "./calendar_main.css"
    current_focused = "CategoryTree"
    is_pop_up = False
    BINDINGS = [
        ("q", "quit", "Quit Nuro"),
        ("u", "show_previous_month", "Show prev month"),
        ("i", "show_next_month", "Show next month"),
        ("y", "show_current_month", "Show current month"),
        ("e", "focus_on_calendar", "Move to calendar"),
        ("w", "focus_on_sidebar", "Move to sidebar"),
        ("ctrl+j", "move_down_to_tag_tree", "Move down to tag tree"),
        ("ctrl+k", "move_up_to_category_tree", "Move up to category tree"),
        ("o", "close_pop_up", "Close pop up box"),
        ("f", "toggle_files", "Toggle Files"),
    ]
    show_sidebar = reactive(True)
    pilot = None

    def on_mount(self):
        self.set_focus(self.query_one(CategoryTree))

    def compose(self):
        yield CalendarApp()

    # Display pop-up box when selecting a cell
    def on_calendar_task_cell_selected(self, event: Calendar.TaskCellSelected):
        cell_events = event.cell_events
        year, month, day = event.year, event.month, event.day
        table = display_events_by_list(cell_events)

        self.query_one(CalendarContainer).mount(
            Vertical(
                Static(
                    Text(
                        f"{day} {calendar.month_name[month]} {year}",
                        style=Style(bold=True, color=TABLE_HEADER_DATE_COLOR),
                    ),
                    classes="task-pop-up-header",
                ),
                Container(
                    Static(table, classes="task-pop-up-table"),
                    classes="task-pop-up-table-container",
                ),
                classes="task-pop-up-container",
            )
        )
        self.is_pop_up = True

    def action_quit(self):
        self.exit()

    def action_show_next_month(self):
        if self.is_pop_up:
            return
        calendar_container = self.query_one(CalendarContainer)
        calendar_container.update_month_by_offset(1)

    def action_show_previous_month(self):
        if self.is_pop_up:
            return
        calendar_container = self.query_one(CalendarContainer)
        calendar_container.update_month_by_offset(-1)

    def action_show_current_month(self):
        if self.is_pop_up:
            return
        calendar_container = self.query_one(CalendarContainer)
        now = datetime.now()
        cur_year, cur_month = now.year, now.month
        calendar_container.update_year_and_month(cur_year, cur_month)

    def action_focus_on_calendar(self):
        if self.is_pop_up:
            return
        self.set_focus(self.query_one(Calendar))
        self.current_focused = "Calendar"

        cat_tree = self.query_one(CategoryTree)
        tag_tree = self.query_one(TagTree)
        calendar_utils.remove_left_arrow_tree(cat_tree.highlighted_node)
        calendar_utils.remove_left_arrow_tree(tag_tree.highlighted_node)
        calendar_utils.remove_highlight(cat_tree.highlighted_node)
        calendar_utils.remove_highlight(tag_tree.highlighted_node)

    def action_focus_on_sidebar(self):
        if self.is_pop_up:
            return
        self.set_focus(self.query_one(CategoryTree))
        self.current_focused = "CategoryTree"

        cal = self.query_one(Calendar)

        if self.is_pop_up:
            return

        cat_tree = self.query_one(CategoryTree)
        calendar_utils.remove_left_arrow(cal.cur_focused_cell)
        calendar_utils.add_highlight(cat_tree.highlighted_node)

        ############# UNKNOWN ERROR - Temporary Fix ############
        ngbrs = [
            self.query_one("#cell0"),
            self.query_one("#cell1"),
            self.query_one("#cell2"),
            self.query_one("#cell3"),
            self.query_one("#cell4"),
            self.query_one("#cell5"),
            self.query_one("#cell6"),
        ]
        for ngbr_cell in ngbrs:
            temp = []
            while ngbr_cell.children:
                child = ngbr_cell.children[0]
                temp.append(child.render())
                child.remove()

            for child in temp:
                ngbr_cell.mount(Static(child))
        ######################################################

    def action_move_down_to_tag_tree(self):
        if self.is_pop_up:
            return
        if self.current_focused != "CategoryTree":
            return
        tag_tree = self.query_one(TagTree)
        self.set_focus(tag_tree)
        self.current_focused = "TagTree"
        category_tree = self.query_one(CategoryTree)
        calendar_utils.remove_highlight(category_tree.highlighted_node)
        calendar_utils.remove_left_arrow_tree(category_tree.highlighted_node)
        calendar_utils.add_left_arrow_tree(tag_tree.highlighted_node)
        calendar_utils.add_highlight(tag_tree.highlighted_node)

    def action_move_up_to_category_tree(self):
        if self.is_pop_up:
            return
        if self.current_focused != "TagTree":
            return
        category_tree = self.query_one(CategoryTree)
        self.set_focus(category_tree)
        self.current_focused = "CategoryTree"
        tag_tree = self.query_one(TagTree)
        calendar_utils.remove_highlight(tag_tree.highlighted_node)
        calendar_utils.remove_left_arrow_tree(tag_tree.highlighted_node)
        calendar_utils.add_left_arrow_tree(category_tree.highlighted_node)
        calendar_utils.add_highlight(category_tree.highlighted_node)

    def action_close_pop_up(self):
        if not self.is_pop_up:
            return
        self.query_one(".task-pop-up-container").remove()
        self.query_one(Calendar).is_pop_up = False
        self.is_pop_up = False

    def action_toggle_files(self):
        self.show_sidebar = not self.show_sidebar
        sidebar_container = self.query_one(SidebarContainer)
        if self.show_sidebar:
            sidebar_container.styles.display = "block"
        else:
            sidebar_container.styles.display = "none"


if __name__ == "__main__":
    app = Entry()
    app.run()



from urllib.parse import urljoin

import requests
from requests import HTTPError

from girok.api.entity import APIResponse
from girok.constants import BASE_URL


def verify_access_token(access_token: str) -> bool:
    resp = requests.get(
        url=urljoin(BASE_URL, "auth/verify/access-token"),
        headers={"Authorization": "Bearer " + access_token},
    )
    return resp.status_code == 200


def send_verification_code(email: str) -> APIResponse:
    resp = requests.post(url=urljoin(BASE_URL, "auth/verification-code"), json={"email": email})

    try:
        resp.raise_for_status()
        return APIResponse(is_success=True)
    except HTTPError as e:
        error_body = resp.json()
        error_message = error_body["message"]
        return APIResponse(is_success=False, error_message=error_message)


def verify_verification_code(email: str, verification_code: str) -> APIResponse:
    resp = requests.post(
        url=urljoin(BASE_URL, "auth/verification-code/check"),
        json={"email": email, "verificationCode": verification_code},
    )

    try:
        resp.raise_for_status()
        return APIResponse(is_success=True)
    except HTTPError as e:
        error_body = resp.json()
        error_message = error_body["message"]
        return APIResponse(is_success=False, error_message=error_message)


def register(email: str, verification_code: str, password: str) -> APIResponse:
    resp = requests.post(
        url=urljoin(BASE_URL, "sign-up"),
        json={
            "email": email,
            "verificationCode": verification_code,
            "password": password,
        },
    )

    try:
        resp.raise_for_status()
        return APIResponse(is_success=True)
    except HTTPError as e:
        try:
            error_body = resp.json()
            error_message = error_body["message"]
        except:
            error_message = "Registration failed"

        return APIResponse(is_success=False, error_message=error_message)


def login(email: str, password: str) -> APIResponse:
    resp = requests.post(url=urljoin(BASE_URL, "login"), json={"email": email, "password": password})

    try:
        resp.raise_for_status()
        return APIResponse(is_success=True, body=resp.json())
    except HTTPError as e:
        try:
            error_body = resp.json()
            error_message = error_body["message"]
        except:
            error_message = "Login failed"

        return APIResponse(is_success=False, error_message=error_message)



import calendar
from datetime import datetime, timedelta
from dataclasses import dataclass
from typing import List, Optional

from girok.constants import CATEGORY_COLOR_PALETTE
from girok.utils.time import convert_iso_date_str_to_date_obj


@dataclass
class EventDate:
    start_date: str
    start_time: Optional[str]
    end_date: Optional[str]
    end_time: Optional[str]


@dataclass
class Repetition:
    repetition_type: str
    repetition_end_date: str


@dataclass
class Category:
    id: int
    name: str


@dataclass
class Event:
    id: int
    name: str
    color_str: str
    tags: List[str]
    priority: str
    memo: str
    event_date: EventDate
    repetition: Repetition
    category_path: List[Category]

    def __post_init__(self):
        self.color_hex = CATEGORY_COLOR_PALETTE[self.color_str]

    def get_all_days_for_month(self, year: int, month: int) -> list[int]:
        """
        1. start_date -> repeat
        2. start_date, start_time -> repeat
        3. start_date, end_date
        4. start_date, start_time, end_date, end_time -> repeat
        """
        month_first_date = datetime(year, month, 1)
        month_last_date = datetime(year, month, calendar.monthrange(year, month)[1])

        start_date = convert_iso_date_str_to_date_obj(self.event_date.start_date)
        end_date = None
        if self.event_date.end_date:
            end_date = convert_iso_date_str_to_date_obj(self.event_date.end_date)

        repetition_type = self.repetition.repetition_type
        repetition_end_date = self.repetition.repetition_end_date

        if repetition_type: # only start_date matters
            if repetition_type == 'DAILY':
                fallin_days: List[int] = []
                start = max(start_date, month_first_date)
                end = month_last_date if not repetition_end_date else min(month_last_date, repetition_end_date)

                return [d for d in range(start.day, end.day + 1)]
            elif repetition_type == 'WEEKLY':
                fallin_days: List[int] = []
                weekday = start_date.weekday()
                diff = (weekday - month_first_date.weekday() + 7) % 7
                month_first_weekday_date = month_first_date + timedelta(days=diff)
                cut_off_date = month_last_date if not repetition_end_date else min(month_last_date, repetition_end_date)

                while month_first_weekday_date <= cut_off_date:
                    fallin_days.append(month_first_weekday_date.day)
                    month_first_weekday_date += timedelta(days=7)

                return fallin_days
            elif repetition_type == 'MONTHLY':
                return [start_date.day]
            elif repetition_type == 'YEARLY':
                return [start_date.day]
            else:
                raise ValueError("Invalid repetition type")
        else:
            if end_date:
                start = max(start_date, month_first_date)
                end = min(end_date, month_last_date)
                return [d for d in range(start.day, end.day + 1)]
            else: # single-day
                return [start_date.day]


from dataclasses import dataclass


@dataclass
class APIResponse:
    is_success: bool
    body: dict = None
    error_message: str = None

    def __post_init__(self):
        if self.is_success:
            if self.error_message:
                raise ValueError("Success response should not have an error message.")
        else:
            if not self.error_message:
                raise ValueError("Failure response requires an error message.")
            if self.body is not None:
                raise ValueError("Failure response should not have a body.")



import shutil

from rich import print
from rich.align import Align
from rich.console import Console
from rich.style import Style
from rich.text import Text

from girok.constants import DisplayArrowType, DisplayBoxType

console = Console()


def center_print(text: str, text_type: DisplayBoxType, wrap: bool = False) -> None:
    style = Style(color=text_type.text_color_hex, bgcolor=text_type.bg_color_hex)

    width = shutil.get_terminal_size().columns // 2 if wrap else shutil.get_terminal_size().columns

    content = Text(text, style=style)
    console.print(Align.center(content, style=style, width=width), height=50)


def arrow_print(text: str, text_type: DisplayArrowType) -> None:
    print(f"[{text_type.value}]> {text}[/{text_type.value}]")



from rich.style import Style
from rich.text import Text
from textual import events, log
from textual.app import App, ComposeResult
from textual.containers import Container, Horizontal, Vertical
from textual.messages import Message
from textual.reactive import reactive, var
from textual.widget import Widget
from textual.widgets import Button, Footer, Header, Label, Placeholder, Static, Tree
from textual.widgets._tree import TreeNode

import girok.calendar_cli.utils as calendar_utils
import girok.api.category as category_api
import girok.api.task as task_api
from girok.constants import CATEGORY_COLOR_PALETTE, Emoji
from girok.calendar_cli.entity import Category



class CategoryTree(Tree):
    CSS_PATH = "./demo_dock.css"
    categories: list[dict] = None
    can_focus = True
    can_focus_children = True
    auto_expand = False
    highlighted_node = None
    selected_node = None
    init_select = False
    init_highlight = False

    class CategoryChanged(Message):
        def __init__(self, category: Category):
            super().__init__()
            self.category = category

    class CustomTestMessage(Message):
        def __init__(self):
            super().__init__()

    def on_mount(self):
        self.highlighted_node = self.root
        self.selected_node = self.root
        calendar_utils.add_left_arrow_tree(self.highlighted_node)

        self.line = 0
        resp = category_api.get_all_categories()
        if not resp.is_success:
            exit(0)
        self.categories = resp.body['rootCategories']
        self.root.expand()

        for category in self.categories:
            top_cat = self.root.add(
                category['name'], expand=True, data={"color": category['color'], "id": category['id']}
            )
            top_cat.allow_expand = True
            calendar_utils.build_category_tree(top_cat, category['children'])

    def on_key(self, evt):
        if evt.key == "j":
            self.action_cursor_down()
        elif evt.key == "k":
            self.action_cursor_up()
        elif evt.key == "o":
            self.action_select_cursor()

    def render_label(self, node, base_style: Style, style: Style):
        node_label = node._label.copy()
        icon = ""
        if node.parent is None:
            icon = "📖 "
        elif node.parent.parent is None:
            icon = Text("● ", style=CATEGORY_COLOR_PALETTE[node.data["color"]])
        text = Text()
        text.append(icon)
        text.append(node_label)
        return text

    def on_tree_node_selected(self, event: Tree.NodeSelected):
        event.stop()
        full_cat_path = calendar_utils.get_full_path_from_node(event.node)
        self.selected_node = event.node
        if full_cat_path == "":
            category = Category(id=None, path="")
        else:
            category = Category(id=event.node.data['id'], path=full_cat_path)
        self.post_message(self.CategoryChanged(category))

    def on_tree_node_highlighted(self, event: Tree.NodeHighlighted):
        event.stop()
        prev_highlighted_node = self.highlighted_node
        calendar_utils.remove_left_arrow_tree(prev_highlighted_node)
        calendar_utils.remove_highlight(prev_highlighted_node)
        calendar_utils.add_left_arrow_tree(event.node)
        calendar_utils.add_highlight(event.node)
        self.highlighted_node = event.node

    def on_focus(self, evt):
        calendar_utils.add_left_arrow_tree(self.highlighted_node)


class TagTree(Tree):
    CSS_PATH = "./demo_dock.css"
    tags = []
    can_focus = True
    can_focus_children = True
    auto_expand = False
    highlighted_node = None
    selected_node = None

    class TagChanged(Message):
        def __init__(self, tag: str):
            super().__init__()
            self.tag = tag

    class CustomTestMessage(Message):
        def __init__(self):
            super().__init__()

    def on_mount(self):
        self.select_node(self.root)
        self.action_select_cursor()
        self.highlighted_node = self.root
        self.selected_node = self.root

        # tags = task_api.get_tags()
        resp = task_api.get_all_tags()
        if not resp.is_success:
            exit(0)
        self.tags = resp.body['tags']
        self.root.expand()

        for tag in self.tags:
            self.root.add(tag, expand=True)

    def on_key(self, evt):
        if evt.key == "j":
            self.action_cursor_down()
        elif evt.key == "k":
            self.action_cursor_up()
        elif evt.key == "o":
            self.action_select_cursor()

    def on_focus(self, evt):
        calendar_utils.add_left_arrow_tree(self.highlighted_node)

    def render_label(self, node, base_style: Style, style: Style):
        node_label = node._label.copy()

        icon = ""
        if node.parent is None:
            icon = "📖 "
        elif node.parent.parent is None:
            icon = Text("● ", style="white")

        text = Text()
        text.append(icon)
        text.append(node_label)
        return text

    def on_tree_node_selected(self, event: Tree.NodeSelected):
        event.stop()
        tag = str(event.node._label)
        if tag.endswith(" " + Emoji.LEFT_ARROW):
            tag = tag[:-2]

        self.post_message(self.TagChanged(tag))
        self.selected_node = event.node

    def on_tree_node_highlighted(self, event: Tree.NodeHighlighted):
        event.stop()
        prev_highlighted_node = self.highlighted_node
        calendar_utils.remove_highlight(prev_highlighted_node)
        calendar_utils.remove_left_arrow_tree(prev_highlighted_node)
        calendar_utils.add_left_arrow_tree(event.node)
        calendar_utils.add_highlight(event.node)
        self.highlighted_node = event.node


class SidebarMainContainer(Vertical):
    CSS_PATH = "./demo_dock.css"

    def compose(self):
        yield CategoryTree("All Categories", id="sidebar")
        yield TagTree("All Tags", id="tag-tree")


class SidebarContainer(Vertical):
    CSS_PATH = "./demo_dock.css"

    def compose(self):
        yield SidebarMainContainer(id="sidebar-main-container")
